using System;
using System.Linq;
using Avalonia.Controls;
using Avalonia.Controls.Shapes;
using Avalonia.Media;

namespace GUI
{
    public class Piece
    {
        private readonly bool _drawGhost;
        private readonly bool _isGhost;
        private readonly Grid _tetrisGrid;
        private int _offsetX;
        private int _offsetY;
        public Piece Ghost;
        public Matrix Matrix;

        public Piece(Grid tetrisGrid, Matrix matrix, int offsetX, int offsetY, bool isGhost = false,
            bool drawGhost = true)
        {
            _tetrisGrid = tetrisGrid;
            Matrix = matrix;
            _offsetX = offsetX;
            _offsetY = offsetY;
            _isGhost = isGhost;
            _drawGhost = drawGhost;
        }

        public void Remove()
        {
            if (!_isGhost) Ghost.Remove();
            foreach (IControl item in
                _tetrisGrid
                    .Children
                    .Where(item => item.Classes.First() == GetHashCode().ToString())
                    .ToList() //ToList is required because the list is being modified.
            ) _tetrisGrid.Children.Remove(item);
        }

        public bool Move(int x, int y)
        {
            Remove();
            _offsetX += x;
            _offsetY += y;
            if (Place()) return true;
            _offsetX -= x;
            _offsetY -= y;
            if (!Place()) throw new Exception("huh???");
            return false;
        }

        public void Rotate(int direction)
        {
            Remove();
            Matrix = direction > 0 ? Matrix.Rotate90() : Matrix.Rotate90CounterClockwise();
            if (Place()) return;
            Matrix = direction < 0 ? Matrix.Rotate90() : Matrix.Rotate90CounterClockwise();
            if (!Place()) throw new Exception("huh???");
        }

        public bool Place()
        {
            int[,] values = Matrix.Value;
            for (var i = 0; i < values.GetLength(0); i++)
            for (var j = 0; j < values.GetLength(1); j++)
            {
                if (values[i, j] == 0) continue;
                if (_tetrisGrid.Children.Select(item => (Control) item).Where(item =>
                        Grid.GetRow(item) == i + _offsetY && Grid.GetColumn(item) == j + _offsetX).ToList().Count >
                    0) return false;
                if (j + _offsetX < 0 || j + _offsetX > TetrisGrid.GridWidth - 1) return false;
                if (i + _offsetY == TetrisGrid.GridHeight) return false;
            }

            if (!_isGhost && _drawGhost)
            {
                Ghost = new Piece(_tetrisGrid, Matrix, _offsetX, _offsetY, true);
                Ghost.Place();
                while (Ghost.Move(0, 1))
                {
                }
            }

            for (var i = 0; i < values.GetLength(0); i++)
            for (var j = 0; j < values.GetLength(1); j++)
            {
                // Als de waarde niet gelijk is aan 1,
                // dan hoeft die niet getekent te worden:
                if (values[i, j] != 1) continue;

                Rectangle rectangle = new()
                {
                    Width = 25, // Breedte van een 'cell' in de Grid
                    Height = 25, // Hoogte van een 'cell' in de Grid
                    Stroke = Brushes.White, // De rand
                    StrokeThickness = 1, // Dikte van de rand
                    Fill = Matrix.Color, // Achtergrondkleur
                    Opacity = _isGhost ? 0.5 : 1,
                    Classes = new Classes(GetHashCode().ToString())
                };

                _tetrisGrid.Children.Add(rectangle); // Voeg de rectangle toe aan de Grid
                Grid.SetRow(rectangle, i + _offsetY); // Zet de rij
                Grid.SetColumn(rectangle, j + _offsetX); // Zet de kolom
            }

            return true;
        }
    }
}