﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.AspNetCore.SignalR;

namespace TetrisServer.Hubs
{
    public class TetrisHub : Hub
    {
        private static readonly Dictionary<string, string> Players = new();
        private static readonly Dictionary<string, Timer> Timers = new();
        private readonly IHubContext<TetrisHub> _hubContext;

        public TetrisHub(IHubContext<TetrisHub> hubContext)
        {
            _hubContext = hubContext;
        }

        public override async Task OnDisconnectedAsync(Exception exception)
        {
            Console.WriteLine("Disconnect from " + Context.ConnectionId);
            var gameId = Players[Context.ConnectionId];
            Players.Remove(Context.ConnectionId);
            if (Players.Any(item => item.Value == gameId) == false)
            {
                await Timers[gameId].DisposeAsync();
                Timers.Remove(gameId);
            }
        }

        public async Task KeyPress(string gameId, int seed, string key)
        {
            await Clients.Group(gameId).SendAsync("KeyPress", seed, key);
        }

        private static int ToInt(string id)
        {
            return BitConverter.ToInt32(Encoding.Default.GetBytes(id));
        }

        public async Task Join(string gameId, string name)
        {
            if (Timers.ContainsKey(gameId))
            {
                await Clients.Caller.SendAsync("AlreadyStarted");
                return;
            }
            await Groups.AddToGroupAsync(Context.ConnectionId, gameId);
            await Clients.Caller.SendAsync("Seed", ToInt(Context.ConnectionId), name);
            foreach (var seed in Players.Where(item => item.Value == gameId).Select(item => item.Key))
                await Clients.Caller.SendAsync("Join", ToInt(seed), name);
            await Clients.OthersInGroup(gameId).SendAsync("Join", ToInt(Context.ConnectionId), name);
            Players.Add(Context.ConnectionId, gameId);
        }

        public void Start(string gameId)
        {
            Timers.Add(gameId, new Timer(_ => { _hubContext.Clients.Group(gameId).SendAsync("Timer"); }, null, 0, 500));
        }
    }
}